﻿/*
jasonTree1.0：多级多选树 
author: http://www.cnblogs.com/smallidea

示例
var json = [
    {
        "id": "f45013c6-6558-9208-8e04-3ac140125208",
        "text": "照明类",
        "isChecked": false,
        "children": [
            {
                "id": "f45013c6-655d-1208-8e04-3ac140125208",
                "text": "照明产品",
                "isChecked": false,
                "children": []
            }
        ]
    }];
        $(function () {
            $('#ss').jasonTree({
                data: json
            });
        });
*/

(function ($) {
    // 插件的定义    
    $.fn.jasonTree = function (options) {
        _debug(this);
        // build main options before element iteration    
        var opts = $.extend({}, $.fn.jasonTree.defaults, options);
        // iterate and reformat each matched element    
        return this.each(function () {
            $this = $(this);
            // build element specific options    
            var o = $.meta ? $.extend({}, opts, $this.data()) : opts;

            // reload html
            var markup = "<div class='jasonTree'><div class='j_nodeList'></div>";
            markup += _getTreeHTMLByData(o.data);
            markup += "</div>";
            $this.after(markup);

            var $target = $this.next(".jasonTree");

            // 更新节点状态，初始化已选
            _init($target, $target.find("> ul > li"));

            // 点击checkbox
            $target.find('span.j_checkbox').bind('click', function () {
                var isChecked = $(this).hasClass("j_checkbox_selected");
                var id = $(this).attr("tag");

                if (isChecked) {
                    _unchecked($target, $(this));
                }
                else { _checked($target, $(this)); }

                _reloadDom($target, $this, $(this));//更新dom      
            });

            // 
            $target.delegate("div.l_selectedNode", "mouseover mouseout click", function (event) {
                if (event.type == 'mouseover') {
                    $(this).addClass("l_selectedNode_hover");
                } else if (event.type == 'mouseout') {
                    $(this).removeClass("l_selectedNode_hover");
                } else if (event.type == "click") {            // 取消已选
                    var id = $(this).attr("tag");
                    var $span = $target.find("li span[tag=" + id + "]");
                    if ($span[0]) {
                        _unchecked($target, $span);//未选

                        $(this).remove();//删除节点

                        _reloadDom($target, $this, $span);//更新dom                   
                    }

                    return;
                }
            });

            var z_index = 999;// 默认值

            // 展开/收起 子节点列表
            $target.delegate('a.j_node_ico', 'mouseover mouseout click', function (event) {
                if (event.type == 'mouseover') {

                } else if (event.type == 'click') {
                    if ($(this).hasClass("j_node_icoPlus")) {// 未展开
                        $(this).removeClass("j_node_icoPlus").addClass("j_node_icoMinus");

                        // 子节点列表
                        var tmpTop = $(this).parent().innerHeight() - 1;
                        var tmpWidth = $(this).parent().innerWidth();
                        $(this).parent().parent().addClass("j_currentParent"); // 防止子的绝对定位无法突破父节点列表的问题
                        $(this).parent().next("ul").show().css({ 'z-index': z_index++, top: tmpTop, display: 'inline-block' });
                        $(this).parent().next("ul")
                            .find("div.j_node_line").attr("style", "z-index:" + (z_index++) + ";width:" + tmpWidth + "px");
                        $(this).parent().addClass("j_node_selected");

                        // 事件监听
                        $(this).parent().parent().bind("mouseleave", _nodeMouseLeaveEvent);
                    }
                    else {// 已展开                        
                        $(this).parent().next("ul").hide("fast", function () {
                            $(this).parent().removeClass("j_currentParent");//在动画完成时，去掉relative
                        });
                        $(this).parent().removeClass("j_node_selected");

                        $(this).removeClass("j_node_icoMinus").addClass("j_node_icoPlus");                        

                        // 去掉事件监听
                        $(this).parent().parent().unbind("mouseleave", _nodeMouseLeaveEvent);
                    }

                    return;
                } 
            });

        });
    };

    // 鼠标完全离开对象事件 的监听
    var _nodeMouseLeaveEvent = function () {
        $(this).find("a.j_node_ico").removeClass("j_node_icoMinus").addClass("j_node_icoPlus");        
        $(this).find("ul").hide("fast", function () {
            $(this).parent().removeClass("j_currentParent");//在动画完成时，去掉relative
        });
        $(this).find("div.j_node").removeClass("j_node_selected");

        $(this).unbind("mouseleave", _nodeMouseLeaveEvent);
    }

    // 选中
    function _checked($target, $span) {
        if (!$span.hasClass("j_checkbox_selected")) {
            //样式
            $span.removeClass("j_checkbox_auto");
            $span.removeClass("j_checkbox_has");
            $span.addClass("j_checkbox_selected");

            //已选列表
            var id = $span.attr("tag");
            var text = $span.next().text();
            _addHTMLBySelectedNode($target, id, text);
        }
    }

    // 取消选中
    function _unchecked($target, $span) {
        if (!$span.hasClass("j_checkbox_auto")) {
            //样式
            $span.removeClass("j_checkbox_selected");
            $span.removeClass("j_checkbox_has");
            $span.addClass("j_checkbox_auto");

            //已选列表
            var id = $span.attr("tag");
            _removeHTMLBySelectedNode($target, id);
        }
    }

    // 部分选中
    function _hasChild($target, $span) {
        if (!$span.hasClass("j_checkbox_has")) {
            // 样式
            $span.removeClass("j_checkbox_auto");
            $span.removeClass("j_checkbox_selected");
            $span.addClass("j_checkbox_has");

            //已选列表
            var id = $span.attr("tag");
            _removeHTMLBySelectedNode($target, id);
        }
    }

    // 初始化
    function _init($target, $liArray) {
        if (!$liArray[0]) return 0;//退出

        var itemTotal = 0;//:not(.j_checkbox_auto)
        $liArray.find("> div span:eq(0)").each(function (i, n) {
            // 在头部显示已选中的节点
            var $self = $(n);
            if ($self.hasClass("j_checkbox_selected")) {
                var id = $self.attr("tag");
                var text = $self.next("span.j_nodeName").text();
                _addHTMLBySelectedNode($target, id, text);
                itemTotal++;
            }

            // 成员有选中，但不是全选时，显示has状态
            var children = $self.parent().parent().find("> ul > li");
            var selectedTotal = _init($target, $(children));
            if (children.length > 0 && selectedTotal < children.length && selectedTotal > 0) {
                _hasChild($target, $self);
            }
        });
        return itemTotal;
    }

    // 更新dom
    function _reloadDom($target, $input, $span) {
        var isSelected = $span.hasClass("j_checkbox_selected");//是否选中

        // 更新子节点
        if (isSelected) {
            $span.parent().parent().find("ul span.j_checkbox").each(function (i, n) {
                _checked($target, $(n));
            });
        } else {
            $span.parent().parent().find("ul span.j_checkbox").each(function (i, n) {
                _unchecked($target, $(n));
            });
        }

        // 更新父节点
        $span.parent().parent().parent().parents("li").each(function (i, n) {
            // 操作项
            var $self = $(n);
            var $selfSpan = $(n).find("span:eq(0)");
            var id = $selfSpan.attr("tag");

            // 选中元素的数量
            var totalItems = 0;
            var totalSelectedItems = 0;
            var items = $self.find("ul:eq(0) > li");
            totalItems = items.length;
            totalSelectedItems = $(items).find("> div:first-child span.j_checkbox_selected").length;

            // 更新状态
            if (totalSelectedItems == 0) {
                _unchecked($target, $selfSpan);

                _removeHTMLBySelectedNode($target, id);
            }
            else if (totalSelectedItems == totalItems) {
                _checked($target, $selfSpan);
            }
            else {
                _hasChild($target, $selfSpan);
            }

        });

        // 更新已选中项列表
        _reloadList($target, $target.find("> ul > li"));
    }

    // 更新已选中项列表
    function _reloadList($target, $liArray) {
        if (!$liArray[0]) return;//退出递归

        $liArray.find("> div span:not(.j_checkbox_auto)").each(function (i, n) {
            var $self = $(n);
            if ($self.hasClass("j_checkbox_selected")) {
                var id = $self.attr("tag");
                var text = $self.next("span.j_nodeName").text();
                _addHTMLBySelectedNode($target, id, text);
            } else {
                _reloadList($target, $self.parent().parent().find("> ul > li"));
            }
        });
    }

    // 私有函数：_debugging    
    function _debug($obj) {
        //if (window.console && window.console.log)
        //    window.console.log('jasonTree selection count: ' + $obj.size());
    };

    // 根据传入的json数据生成树的HTML
    function _getTreeHTMLByData(data, isLeaf) {
        if (data == undefined || data == null) return;

        var treehtmlarr = [];

        if (isLeaf == true) {
            treehtmlarr.push("<ul class='j_child'>");
        } else {
            treehtmlarr.push("<ul class='j_root'>");
        }
        if (isLeaf) treehtmlarr.push("<div class='j_node_line'></div>");

        // 节点列表
        for (var i = 0; i < data.length; i++) {
            var text = data[i].text;
            var id = data[i].id;
            var isChecked = data[i].isChecked == true || data[i].isChecked == 1;
            var children = data[i].children;
            var hasChild = children != undefined && children.length > 0;

            treehtmlarr.push("<li>");
            if (hasChild) {
                treehtmlarr.push("<div class='j_node'><a class='j_node_ico j_node_icoPlus'></a>");
            } else {
                treehtmlarr.push("<div class='j_node j_node_hasNoChild'>");
            }

            if (isChecked) {
                treehtmlarr.push("<span tag='" + id + "' class='j_checkbox  j_checkbox_selected' />");
            } else {
                treehtmlarr.push("<span tag='" + id + "' class='j_checkbox  j_checkbox_auto' />");
            }
            treehtmlarr.push("<span class='j_nodeName'>" + text + "</span></div>");

            if (hasChild) {
                treehtmlarr.push(_getTreeHTMLByData(children, true));
            }
            treehtmlarr.push("</li>");
        }

        treehtmlarr.push("</ul>");
        return treehtmlarr.join('');// 转换为string
    };

    // 获取已选节点的HTML
    function _addHTMLBySelectedNode($target, id, text) {
        if (!$target.find("div.j_nodeList div[tag='" + id + "']")[0]) {//是否已存在节点
            //添加节点
            var html = "<div tag='" + id + "' class='l_selectedNode'>" + text + "<a class='j_close'>X</a></div>";
            $target.find("div.j_nodeList").append(html);

            //保存值
            _reloadInput($target, id, true);
        }
    }

    // 删除
    function _removeHTMLBySelectedNode($target, id) {
        var $tmpObj = $target.find("div.j_nodeList div[tag='" + id + "']");
        if ($tmpObj.length == 1) {
            // 删除节点
            $tmpObj.remove();

            // 保存值
            _reloadInput($target, id, false);
        }
    }

    // 更新表单项中的值
    function _reloadInput($target, id, isAdd) {
        // 初始化信息
        var val = $target.prev("input").val();
        var array = [];
        if (val != "" && val != undefined) {
            array = val.split(',');
            if (array[0] == "") array.splice(0, 1);
            if (array[array.length - 1] == "") array.splice(array.length - 1, 1);
        }

        //获取元素的序号
        var index = -1;
        for (var i = 0; i < array.length; i++) {
            if (array[i] == id) {
                index = i;
                break;
            }
        }

        //添加 或 删除
        if (isAdd) {
            if (index < 0) {
                array.push(id);
            } else { return; }
        }
        else {
            if (index > -1) {
                array.splice(index, 1);
            }
        }

        array.sort();//默认的排序方法
        $target.prev("input").val(array.join(','));
    }

    // 暴露的方法
    $.fn.jasonTree.getSelected = function (txt) {
        return '<strong>' + txt + '</strong>';
    };

    // 点击事件
    $.fn.jasonTree.prototype = {
        onCheck: function () {
        }
    };

    // 插件的defaults
    $.fn.jasonTree.defaults = {
        data: null
    };

})(jQuery);//闭包结束